from django.contrib import auth
from django.http import HttpResponseServerError
from django.http.response import HttpResponseBadRequest
from django.shortcuts import redirect
from django.views import View
from requests_oauthlib import OAuth2Session

from accounts.settings import accounts_settings


class LoginView(View):
    def get(self, request):
        return_to = request.GET.get('next')
        if not return_to:
            return HttpResponseBadRequest('Invalid next parameter')
        request.session.__setitem__('next', return_to)
        if not request.user.is_authenticated:
            platform = OAuth2Session(
                accounts_settings.CLIENT_ID,
                scope=accounts_settings.SCOPE,
                redirect_uri=accounts_settings.REDIRECT_URI
            )
            authorization_url, state = platform.authorization_url(
                accounts_settings.PLATFORM_URL + '/accounts/authorize/'
            )
            response = redirect(authorization_url)
            request.session.__setitem__('state', state)
            return response
        return redirect(request.session.pop('next'))


class CallbackView(View):
    def get(self, request):
        state = request.session.pop('state')
        platform = OAuth2Session(accounts_settings.CLIENT_ID, redirect_uri=accounts_settings.REDIRECT_URI, state=state)
        token = platform.fetch_token(
            accounts_settings.PLATFORM_URL + '/accounts/token/',
            client_secret=accounts_settings.CLIENT_SECRET,
            authorization_response=request.build_absolute_uri()
        )
        platform = OAuth2Session(accounts_settings.CLIENT_ID, token=token)
        access_token = token['access_token']
        introspect_url = accounts_settings.PLATFORM_URL + '/accounts/introspect/'
        user_props = platform.post(introspect_url, data={'token': access_token}).json()['user']
        user_props['pennid'] = int(user_props['pennid'])
        user = auth.authenticate(request, remote_user=user_props)
        if user:
            auth.login(request, user)
            return redirect(request.session.pop('next'))
        return HttpResponseServerError()


class LogoutView(View):
    def get(self, request):
        auth.logout(request)
        return redirect(request.GET.get('next', '/'))



from django.contrib import admin
from django.shortcuts import redirect
from django.urls import reverse

from accounts.settings import accounts_settings


class LabsAdminSite(admin.AdminSite):
    """
    Custom admin site that redirects users to log in through platform
    instead of logging in with a username and password
    """
    def login(self, request, extra_context=None):
        if not request.user.is_authenticated:
            return redirect(reverse('accounts:login') + '?next=' + request.GET.get('next'))
        return super().login(request, extra_context)


if accounts_settings.CUSTOM_ADMIN:
    """
    Replace the default admin site with a custom one to log in users through platform.
    Also copy all registered models from the default admin site
    """
    labs_admin = LabsAdminSite()
    labs_admin._registry = admin.site._registry
    admin.site = labs_admin



from django.contrib.auth import get_user_model
from django.contrib.auth.backends import RemoteUserBackend

from accounts.settings import accounts_settings


class LabsUserBackend(RemoteUserBackend):
    def authenticate(self, request, remote_user):
        """
        Authenticate a user given a dictionary of user information from
        platform.
        """
        if not remote_user:
            return
        User = get_user_model()
        user, created = User.objects.get_or_create(
            id=remote_user['pennid'],
            defaults={'username': remote_user['username']}
        )

        if created:
            user.set_unusable_password()
            user.save()
            try:
                user = self.configure_user(request, user)
            except TypeError:
                user = self.configure_user(user)

        # Update user fields if changed
        for field in ['first_name', 'last_name', 'username', 'email']:
            if getattr(user, field) is not remote_user[field]:
                setattr(user, field, remote_user[field])

        # Set or remove admin permissions
        if accounts_settings.ADMIN_PERMISSION in remote_user['product_permission']:
            if not user.is_staff:
                user.is_staff = True
                user.is_superuser = True
        else:
            if user.is_staff:
                user.is_staff = False
                user.is_superuser = False

        user.save()
        self.post_authenticate(user, created, remote_user)
        return user if self.user_can_authenticate(user) else None

    def post_authenticate(self, user, created, dictionary):
        """
        Post Authentication method that is run after logging in a user.
        This allows products to add custom configuration by subclassing
        LabsUserBackend and modifying this method.
        By default this does nothing.
        """
        pass



import os

from django.conf import settings


USER_SETTINGS = getattr(settings, 'PLATFORM_ACCOUNTS', {})

DEFAULTS = {
    'CLIENT_ID': os.environ.get('LABS_CLIENT_ID'),
    'CLIENT_SECRET': os.environ.get('LABS_CLIENT_SECRET'),
    'REDIRECT_URI': os.environ.get('LABS_REDIRECT_URI'),
    'SCOPE': ['read', 'introspection'],
    'PLATFORM_URL': 'https://platform.pennlabs.org',
    'ADMIN_PERMISSION': 'example_admin',
    'CUSTOM_ADMIN': True
}


class AccountsSettings(object):
    """
    Based on https://github.com/encode/django-rest-framework/blob/master/rest_framework/settings.py
    """
    def __init__(self, settings=None, defaults=None):
        self.settings = settings or {}
        self.defaults = defaults or {}

    def __getattr__(self, attr):
        if attr not in self.defaults.keys():
            raise AttributeError('Invalid Penn Labs accounts setting: %s' % attr)

        try:
            val = self.settings[attr]
        except KeyError:
            val = self.defaults[attr]

        setattr(self, attr, val)
        return val


accounts_settings = AccountsSettings(USER_SETTINGS, DEFAULTS)



from django.urls import path

from accounts.views import CallbackView, LoginView, LogoutView


app_name = 'accounts'


urlpatterns = [
    path('callback/', CallbackView.as_view(), name='callback'),
    path('login/', LoginView.as_view(), name='login'),
    path('logout/', LogoutView.as_view(), name='logout'),
]



import os
import sys

from setuptools import setup
from setuptools.command.install import install


VERSION = '0.4.2'
DESCRIPTION = open('README.md', encoding='utf-8').read()


class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version"""
    description = 'verify that the git tag matches our version'

    def run(self):
        tag = os.getenv('CIRCLE_TAG')

        if tag != VERSION:
            info = 'Git tag: {0} does not match the version of this app: {1}'.format(
                tag, VERSION
            )
            sys.exit(info)


setup(
    name='django-labs-accounts',
    version=VERSION,
    packages=['accounts'],
    url='https://github.com/pennlabs/django-labs-accounts',
    project_urls={
        'Changelog': ('https://github.com/pennlabs/django-labs-accounts/blob/master/CHANGELOG.md')
    },
    license='MIT',
    author='Penn Labs',
    author_email='admin@pennlabs.org',
    description='Reusable Django app for Penn Labs accounts',
    long_description=DESCRIPTION,
    long_description_content_type='text/markdown',
    install_requires=[
        'django>=2.0.0',
        'requests-oauthlib>=1.2.0'
    ],
    classifiers=[
        'Framework :: Django',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    python_requires='>=3.5',
    cmdclass={
        'verify': VerifyVersionCommand,
    }
)
